@@ -13,14 +13,14 @@ typedef enum {
 } status;
 
 struct sock {
-    status ( *connect)(connection *);
+    status ( *connect)(connection *, char *);
     status (   *close)(connection *);
     status (    *read)(connection *, size_t *);
     status (   *write)(connection *, char *, size_t, size_t *);
     size_t (*readable)(connection *);
 };
 
-status sock_connect(connection *);
+status sock_connect(connection *, char *);
 status sock_close(connection *);
 status sock_read(connection *, size_t *);
 status sock_write(connection *, char *, size_t, size_t *);
