@@ -8,21 +8,6 @@
 
 #include "ssl.h"
 
-static pthread_mutex_t *locks;
-
-static void ssl_lock(int mode, int n, const char *file, int line) {
-    pthread_mutex_t *lock = &locks[n];
-    if (mode & CRYPTO_LOCK) {
-        pthread_mutex_lock(lock);
-    } else {
-        pthread_mutex_unlock(lock);
-    }
-}
-
-static unsigned long ssl_id() {
-    return (unsigned long) pthread_self();
-}
-
 SSL_CTX *ssl_init() {
     SSL_CTX *ctx = NULL;
 
@@ -30,20 +15,11 @@ SSL_CTX *ssl_init() {
     SSL_library_init();
     OpenSSL_add_all_algorithms();
 
-    if ((locks = calloc(CRYPTO_num_locks(), sizeof(pthread_mutex_t)))) {
-        for (int i = 0; i < CRYPTO_num_locks(); i++) {
-            pthread_mutex_init(&locks[i], NULL);
-        }
-
-        CRYPTO_set_locking_callback(ssl_lock);
-        CRYPTO_set_id_callback(ssl_id);
-
-        if ((ctx = SSL_CTX_new(SSLv23_client_method()))) {
-            SSL_CTX_set_verify(ctx, SSL_VERIFY_NONE, NULL);
-            SSL_CTX_set_verify_depth(ctx, 0);
-            SSL_CTX_set_mode(ctx, SSL_MODE_AUTO_RETRY);
-            SSL_CTX_set_session_cache_mode(ctx, SSL_SESS_CACHE_CLIENT);
-        }
+    if ((ctx = SSL_CTX_new(SSLv23_client_method()))) {
+        SSL_CTX_set_verify(ctx, SSL_VERIFY_NONE, NULL);
+        SSL_CTX_set_verify_depth(ctx, 0);
+        SSL_CTX_set_mode(ctx, SSL_MODE_AUTO_RETRY);
+        SSL_CTX_set_session_cache_mode(ctx, SSL_SESS_CACHE_CLIENT);
     }
 
     return ctx;
