@@ -5,7 +5,7 @@
 
 SSL_CTX *ssl_init();
 
-status ssl_connect(connection *);
+status ssl_connect(connection *, char *);
 status ssl_close(connection *);
 status ssl_read(connection *, size_t *);
 status ssl_write(connection *, char *, size_t, size_t *);
